@@ -42,6 +42,9 @@ class DjangoModelManager(ModelManager):
             # is it mandatory?
             if not field.blank:
                 fields_info.mandatory |= {field.name, field.attname}
+            # is it nullable?
+            if field.null:
+                fields_info.nullable |= {field.name, field.attname}
             # is it a foreign key?
             if isinstance(field, django.db.models.fields.related.ForeignKey):
                 # field to which the foreign key is referring
@@ -69,9 +72,13 @@ class DjangoModelManager(ModelManager):
         # return result
         return fields_info
 
+    def get_table_name(self):
+        return self.orm_model._meta.db_table
+
     # CRUD operations on ORM model instances
 
-    def create_one(self, authenticated_user, graphql_path, graphql_selection=None, ensure_permission=True, **data):
+    def create_one(self, authenticated_user, graphql_path, graphql_selection=None, ensure_permission=True,
+            depth=0, **data):
         # related things
         related_data = {}
         for field_name in list(data.keys()):
@@ -83,6 +90,7 @@ class DjangoModelManager(ModelManager):
                     authenticated_user = authenticated_user,
                     graphql_path = graphql_path + [field_name],
                     ensure_permission = False,
+                    depth = depth + 1,
                     **data.pop(field_name))
                 data[field_name] = foreign_instance
             elif field_name in self.fields_info.related:
@@ -99,7 +107,7 @@ class DjangoModelManager(ModelManager):
             authenticated_user = authenticated_user,
             data = custom_fields_data)
         # pre-save trigger
-        self.model_config.on_before_operation(instance, authenticated_user, Operation.CREATE, data)
+        self.model_config.on_before_operation(instance, authenticated_user, Operation.CREATE, data, depth)
         # enforce permissions
         if ensure_permission:
             self.model_config.ensure_permission(
@@ -127,6 +135,7 @@ class DjangoModelManager(ModelManager):
                     authenticated_user = authenticated_user,
                     graphql_path = graphql_path + [field_name, related_index],
                     ensure_permission = False,
+                    depth = depth + 1,
                     **related_data)
                 getattr(instance, field_name).add(related_instance)
         # validation
@@ -136,7 +145,7 @@ class DjangoModelManager(ModelManager):
         except django.core.exceptions.ValidationError as exception:
             reraise_django_validation_error(graphql_path, exception)
         # post-save trigger
-        self.model_config.on_after_operation(instance, authenticated_user, Operation.CREATE, data)
+        self.model_config.on_after_operation(instance, authenticated_user, Operation.CREATE, data, depth)
         # result
         if graphql_selection is None:
             return instance
@@ -184,7 +193,7 @@ class DjangoModelManager(ModelManager):
         ]
 
     def update_one(self, authenticated_user, graphql_path, graphql_selection=None,
-            _=None, **filters):
+            _=None, depth=0, **filters):
         # variable that contains new data
         data = _ or {}
         # retrieve the instance to update
@@ -202,7 +211,7 @@ class DjangoModelManager(ModelManager):
             data = data,
         )
         # pre-update trigger
-        self.model_config.on_before_operation(instance, authenticated_user, Operation.UPDATE, data)
+        self.model_config.on_before_operation(instance, authenticated_user, Operation.UPDATE, data, depth)
         # related things
         related_data = {}
         for field_name in list(data.keys()):
@@ -220,6 +229,7 @@ class DjangoModelManager(ModelManager):
                         data[field_name] = child_model_config.orm_model_manager.create_one(
                             authenticated_user = authenticated_user,
                             graphql_path = graphql_path + [field_name],
+                            depth = depth + 1,
                             **child_data)
                     # if identifier provided, update existing instance
                     else:
@@ -227,6 +237,7 @@ class DjangoModelManager(ModelManager):
                             authenticated_user = authenticated_user,
                             graphql_path = graphql_path + [field_name],
                             _ = child_data,
+                            depth = depth + 1,
                             **{child_model_config.orm_model_manager.fields_info.primary:
                                 child_identifier})
             # related fields
@@ -267,6 +278,7 @@ class DjangoModelManager(ModelManager):
                         authenticated_user = authenticated_user,
                         graphql_path = graphql_path + [field_name, child_index],
                         _ = child_data,
+                        depth = depth + 1,
                         **{child_model_config.orm_model_manager.fields_info.primary:
                             child_identifier})
                     # store identifier
@@ -284,6 +296,7 @@ class DjangoModelManager(ModelManager):
                     child_instance = child_model_config.orm_model_manager.create_one(
                         authenticated_user = authenticated_user,
                         graphql_path = graphql_path + [field_name, child_index],
+                        depth = depth + 1,
                         **dict({related_field.value_field_name: instance.pk}, **child_data))
                     getattr(instance, field_name).add(child_instance)
         # validation (raise an easy_graphql_server exception instead of a Django one)
@@ -294,7 +307,7 @@ class DjangoModelManager(ModelManager):
         except django.core.exceptions.ValidationError as exception:
             reraise_django_validation_error(graphql_path, exception)
         # post-update trigger
-        self.model_config.on_after_operation(instance, authenticated_user, Operation.UPDATE, data)
+        self.model_config.on_after_operation(instance, authenticated_user, Operation.UPDATE, data, depth)
         # result
         if graphql_selection is None:
             return instance
@@ -324,9 +337,9 @@ class DjangoModelManager(ModelManager):
         else:
             pre_result = {}
         # actually perform operations
-        self.model_config.on_before_operation(instance, authenticated_user, Operation.DELETE)
+        self.model_config.on_before_operation(instance, authenticated_user, Operation.DELETE, None, 0)
         instance.delete()
-        self.model_config.on_after_operation(instance, authenticated_user, Operation.DELETE)
+        self.model_config.on_after_operation(instance, authenticated_user, Operation.DELETE, None, 0)
         # compute & return result
         result = self._instance_to_dict(
             authenticated_user = authenticated_user,
@@ -376,9 +389,9 @@ class DjangoModelManager(ModelManager):
             raise exceptions.NotFoundError(filters) from error
 
     def _instance_to_dict(self, instance, authenticated_user, graphql_selection, graphql_path,
-            ensure_permission=True):
+            ensure_permission=True, depth=0):
         # pre-read trigger
-        self.model_config.on_before_operation(instance, authenticated_user, Operation.READ)
+        self.model_config.on_before_operation(instance, authenticated_user, Operation.READ, None, depth)
         # enforce permissions when requested
         if ensure_permission:
             self.model_config.ensure_permission(
@@ -415,6 +428,7 @@ class DjangoModelManager(ModelManager):
                             graphql_selection = graphql_subselection,
                             graphql_path = graphql_path + [field_name, child_index],
                             ensure_permission = False,
+                            depth = depth + 1,
                         )
                         for child_index, child_instance
                         in enumerate(children_instances)
@@ -430,9 +444,10 @@ class DjangoModelManager(ModelManager):
                     graphql_selection = graphql_subselection,
                     graphql_path = graphql_path + [field_name],
                     ensure_permission = ensure_permission,
+                    depth = depth + 1,
                 )
         # post-read trigger
-        self.model_config.on_after_operation(instance, authenticated_user, Operation.READ)
+        self.model_config.on_after_operation(instance, authenticated_user, Operation.READ, None, depth)
         return result
 
     def build_queryset(self, graphql_selection, authenticated_user):
@@ -502,11 +517,12 @@ class DjangoModelManager(ModelManager):
                     orm_model = related_field.orm_model)
                 prefetch_related.append(
                     django.db.models.Prefetch(
-                        field_name,
+                        f'{field_prefix}{field_name}',
                         queryset = related_model_config.orm_model_manager.build_queryset(
-                            graphql_selection = dict({related_field.value_field_name: None},
-                                **graphql_subselection),
                             authenticated_user = authenticated_user,
+                            graphql_selection = dict(
+                                {related_field.value_field_name: None},
+                                **graphql_subselection),
                         )
                     )
                 )
@@ -547,7 +563,7 @@ class DjangoModelManager(ModelManager):
         # enum
         if getattr(field, 'choices', None):
             graphql_type = conversion.to_graphql_enum_from_choices(
-                prefix = f'{self.model_config.name}__{field.name}',
+                prefix = f'{self.model_config.types_name}__{field.name}',
                 choices = field.choices,
                 schema = self.model_config.schema,
             )
